/*
 *  Motion Gesture Recognition Module
 * 
 *  Copyright (c) 2010-2012
 *  Nintendo Co., Ltd.
 *  All rights reserved.
 *
 * - API definition header file
 *
 */

#ifndef MW_MGR_MGR_H_
#define MW_MGR_MGR_H_

#include <nn/types.h>

/*------------------------------------------------------------------------------------*/
#define		MGRCG_BDYMAX		(    4 )			/* Maximum number of recognized moving bodies				  */
/*------------------------------------------------------------------------------------*/
#define		MGIMG_WIDMIN		(  128 )			/* Minimum value for the original image width				  */
#define		MGIMG_WIDMAX		(  160 )			/* Maximum value for the original image width				  */
#define		MGIMG_HEIMIN		(   96 )			/* Minimum value for original image height				  */
#define		MGIMG_HEIMAX		(  120 )			/* Maximum value for original image height				  */
#define		MGEXT_RECTMIN		(   64 )			/* Minimum value for height width of the rectangular region targeted for processing */
/*------------------------------------------------------------------------------------*/
#define		MGPAR_MODEMIN		(    0 )			/* Minimum value for the recognition movement mode			  */
#define		MGPAR_MODEMAX		(    1 )			/* Maximum value for the recognition movement mode			  */
/*------------------------------------------------------------------------------------*/
#define		MGPAR_FRMMIN		(    0 )			/* Minimum value for the difference sensitivity between frames		  */
#define		MGPAR_FRMMAX		(   10 )			/* Maximum value for the difference sensitivity between frames		  */
/*------------------------------------------------------------------------------------*/
#define		MGPAR_SPDMIN		(    0 )			/* Minimum value for speed sensitivity				  */
#define		MGPAR_SPDMAX		(    4 )			/* Maximum value for speed sensitivity				  */
/*------------------------------------------------------------------------------------*/
#define		MGPAR_SKNMIN		(    0 )			/* Minimum value for flesh-color filter sensitivity			  */
#define		MGPAR_SKNMAX		(  200 )			/* Maximum value for flesh-color filter sensitivity			  */
/*------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------*/
#define		PSERR_TRUE			(    0 )			/* Processing successful						  */
#define		PSERR_MEMR			(   -1 )			/* Insufficient memory.					  */
#define		PSERR_ARGU			(   -2 )			/* Argument error						  */
#define		PSERR_IMAG			(   -3 )			/* Abnormal image						  */
#define		PSERR_PROC			(   -4 )			/* Processing is impossible						  */
#define		PSERR_RECG			(   -5 )			/* Recognition process failed					  */
/*------------------------------------------------------------------------------------*/

namespace mw { namespace mgr {

typedef struct {

	u8			Z ;								/* Dummy						  */
    u8			B ;								/* Pixel Value Green Component 				  */
    u8			G ;								/* Pixel Value Red Component 				  */
    u8			R ;								/* Pixel Value Blue Component    				  */

} RGBColor;

typedef struct {

    s32			xSize  ;						/* Image data width				  */
    s32			ySize  ;						/* Image data height				  */
    RGBColor	*pcImg ;						/* Color image data				  */

} PSCimg ;

typedef struct {

    s32			x ;								/* Point x coordinate [pixel]				  */
    s32			y ;								/* Point y coordinate [pixel]				  */

} PSPoint ;

typedef struct {

	s32			sx ;							/* Rectangular region start point upper left x coordinate [pixel]  */
	s32			sy ;							/* Rectangular region start point upper left y coordinate [pixel]  */
	s32			ex ;							/* Rectangular region end point lower right x coordinate [pixel]  */
	s32			ey ;							/* Rectangular region end point lower right y coordinate [pixel]  */

} PSRect ;

typedef struct {

	u32	        id     ;						/* Moving Body ID						  */
    PSRect		rect   ;						/* Moving Body Rectangle Position					  */
    PSPoint		center ;						/* Moving Body Center Position					  */

} MGObject ;

typedef struct {

    u32         bodynum                ;		/* Number of moving bodies						  */
    MGObject	mgobject[MGRCG_BDYMAX] ;		/* Moving body							  */
	s32         reserve0               ;		/* Reserve 0						  */
	s32			reserve1               ;		/* Reserve 1						  */
	s32			reserve2               ;		/* Reserve 2						  */
	s32			reserve3               ;		/* Reserve 3						  */

} MGResult ;

typedef struct {

	s32			xsize    ;						/* Image data width (128,160)		  */
	s32			ysize    ;						/* Image data height ( 96,120)		  */
    PSRect		mgrect   ; 						/* Rectangular region targeted for processing				  */
    u8          mgmode   ;						/* Recognition movement mode (0,1)			  */
    u8          frmparam ;						/* Difference sensitivity between frames (0 through 10)	  */
    u8	        spdparam ;						/* Speed sensitivity (0,1,2,3,4)			  */
	u8          sknparam ;						/* Flesh-color filter parameter (0 through 200)		  */
    u8          reserve0 ;						/* Reserve Flag 0					  */
	u8          reserve1 ;						/* Reserve Flag 1					  */
	u8          reserve2 ;						/* Reserve Flag 2					  */
	u8          reserve3 ;						/* Reserve Flag 3					  */

} MGConfig ;

s32 		MGRcgGetWorkSize		( MGConfig*, u32*     ) ;
s32 		MGRcgProcessInit		( MGConfig*, u8*     ) ;
s32 		MGRcgProcessExec		( MGConfig*, PSCimg*, MGResult* ) ;


}}

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {

    s32			x ;								/* Point x coordinate [pixel]				  */
    s32			y ;								/* Point y coordinate [pixel]				  */

} mwmgrPSPoint ;

typedef struct {

	s32			sx ;							/* Rectangular region start point upper left x coordinate [pixel]  */
	s32			sy ;							/* Rectangular region start point upper left y coordinate [pixel]  */
	s32			ex ;							/* Rectangular region end point lower right x coordinate [pixel]  */
	s32			ey ;							/* Rectangular region end point lower right y coordinate [pixel]  */

} mwmgrPSRect ;

typedef struct {

	u8			Z ;								/* Dummy						  */
    u8			B ;								/* Pixel Value Green Component 				  */
    u8			G ;								/* Pixel Value Red Component 				  */
    u8			R ;								/* Pixel Value Blue Component    				  */

} mwmgrRGBColor;

typedef struct {

    s32				xSize  ;					/* Image data width				  */
    s32				ySize  ;					/* Image data height				  */
    mwmgrRGBColor	*pcImg ;					/* Color image data				  */

} mwmgrPSCimg ;

typedef struct {

	u32				id     ;					/* Moving Body ID						  */
    mwmgrPSRect		rect   ;					/* Moving Body Rectangle Position					  */
    mwmgrPSPoint	center ;					/* Moving Body Center Position					  */

} mwmgrMGObject ;

typedef struct {

    u32				bodynum                ;	/* Number of moving bodies						  */
    mwmgrMGObject	mgobject[MGRCG_BDYMAX] ;	/* Moving body							  */
	s32				reserve0               ;	/* Reserve 0						  */
	s32				reserve1               ;	/* Reserve 1						  */
	s32				reserve2               ;	/* Reserve 2						  */
	s32				reserve3               ;	/* Reserve 3						  */

} mwmgrMGResult ;

typedef struct {

	s32				xsize    ;					/* Image data width (128,160)		  */
	s32				ysize    ;					/* Image data height ( 96,120)		  */
    mwmgrPSRect		mgrect   ; 					/* Rectangular region targeted for processing				  */
    u8				mgmode   ;					/* Recognition movement mode (0,1)			  */
    u8				frmparam ;					/* Difference sensitivity between frames (0 through 10)		  */
    u8				spdparam ;					/* Speed sensitivity (0,1,2,3,4)			  */
	u8				sknparam ;					/* Flesh-color filter parameter (0 through 200)		  */
    u8				reserve0 ;					/* Reserve Flag 0					  */
	u8				reserve1 ;					/* Reserve Flag 1					  */
	u8				reserve2 ;					/* Reserve Flag 2					  */
	u8				reserve3 ;					/* Reserve Flag 3					  */

} mwmgrMGConfig ;

s32 	mwmgrMGRcgGetWorkSize		( mwmgrMGConfig*, u32*     ) ;
s32 	mwmgrMGRcgProcessInit		( mwmgrMGConfig*, u8*     ) ;
s32		mwmgrMGRcgProcessExec		( mwmgrMGConfig*, mwmgrPSCimg*, mwmgrMGResult* ) ;

#ifdef __cplusplus
}
#endif

#endif /* MW_MGR_MGR_H_ */

