﻿/*---------------------------------------------------------------------------*
  Project:  Horizon
  File:     demo_CommandCache.h

  Copyright (C)2009-2013 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Rev: 50863 $
 *---------------------------------------------------------------------------*/

#ifndef DEMO_COMMAND_CACHE_H_
#define DEMO_COMMAND_CACHE_H_

#include "demo/Utility/demo_Utility.h"

namespace demo
{

    namespace detail 
    {        
        /* Please see man pages for details 
        
   
        
        */
        struct CommandBuffer3d
        {        
            unsigned data : 32; // Data to write to register
            unsigned addr : 16; // Address of register to write to
            unsigned be   : 4;  // Byte-enabled
            unsigned size : 7;  // Number of data items - 1. If 0, single access; if 1 or more, burst access.
            unsigned rsv  : 4;  // Reserved region
            unsigned seq  : 1;  // Access mode for burst access
                                // If 0, write to single register; if 1, write to successive registers */
        };

        // Address of register for configuring index of vertex shader floating-point register
        const u32 VS_UNIFORM_FLOAT_REG_INDEX = 0x2c0;
        // Start address of register for writing vertex shader floating-point uniform value
        const u32 VS_UNIFORM_FLOAT_REG_VALUE_BEGIN = 0x2c1;
        // End address of register for writing vertex shader floating-point uniform value
        const u32 VS_UNIFORM_FLOAT_REG_VALUE_END = 0x2c8;
    
        // Number of vertex shader floating-point registers
        const u32 VS_UNIFORM_FLOAT_REG_NUM = 96;

        // Max number when using vertex shader floating-point registers in 4x4 matrix
        const u32 VS_UNIFORM_FLOAT_REG_MATRIX_NUM = VS_UNIFORM_FLOAT_REG_NUM / 16;

        /* Please see man pages for details 
        
   
        
        */
        class VertexShaderUniformMatrix : 
            private nn::util::NonCopyable<VertexShaderUniformMatrix>
        {
        public:
            VertexShaderUniformMatrix(void);
            virtual ~VertexShaderUniformMatrix();
        public:
            void Initialize(void);
            void Finalize(void);

        public:
            bool m_Valid;
            bool m_Padding[3];

            // Index of vertex shader floating point register
            u32 m_RegisterIndex;
            // Offset into 3D command buffer
            u32 m_Offset[4][4];
        };
        
        void GetVSUniformOffset(u8* bufferAddress, const u32 startOffset,
            const GLsizei bufferSize, const u32 uniformFloatRegIndex, u32* uniformOffsetArray);
    }

    enum
    {
        COMMAND_CACHE_UNINITIALIZED, // Command list uninitialized
        COMMAND_CACHE_INITIALIZED,   // Command list initialized
        COMMAND_CACHE_FINALIZED,     // Command list finalized
        COMMAND_CACHE_BEGIN_SAVE,    // Command list saving begun
        COMMAND_CACHE_END_SAVE       // Command list saving finished
    };

    const u32 DEMO_MAX_COMMAND_CACHE_MODELS_NUM = 128;
    const u32 DEMO_MAX_COMMAND_CACHE_MATRIX_NUM = demo::detail::VS_UNIFORM_FLOAT_REG_MATRIX_NUM;
    
    /* Please see man pages for details 
    
   
    
    */
    class CommandCache : private nn::util::NonCopyable<CommandCache>
    {
    public:    
        CommandCache(void);
        virtual ~CommandCache(void);

    public:
        void Initialize(const u32 CommandListSize, const u32 RequestNum,            
            const GLboolean copyCommandBuffer, const GLbitfield stateMask,
            const u32 totalModelNum = 1);
        void Finalize(void);

    public:
        // Save the vertex shader floating-point register index to the specified array index
        void SetVSUniformMatrixRegisterIndex(const u32 modelIndex, const u32 uniformMatrixIndex, const u32 regsiterIndex);

    public:
        // Start saving command list.
        void BeginSave(void);
        // Save offset to start of command list for rendering one model.
        void SaveCommandBufferStartOffset(const u32 modelIndex);
        // Stop saving command list.
        void EndSave(void);

    public:
        // Update the vertex shader floating-point register matrix value as indicated by the array index of the specified matrix.
        void UpdateVSUniformMatrix(const u32 modelIndex, const u32 uniformMatrixIndex, const nn::math::MTX44& matrix);
        // Add this saved command list to the current command list.
        void Append(void);    

    public:
        void Print(void);
    
    protected:
        u32 m_State;

        GLsizei m_CommandListSize;
        GLsizei m_CommandListRequestNum;
        GLboolean m_CopyCommandBuffer;
        GLboolean m_Padding[3];
        GLbitfield m_StateMask;

        u32 m_TotalModelNum;
        u32 m_ModelIndex;

        GLuint m_CurrentCommandListId;
        GLuint m_CommandListId;        
        GLint m_CommandBufferStartOffsetArray[DEMO_MAX_COMMAND_CACHE_MODELS_NUM];
        GLuint m_CommandBufferOffset;
        GLsizei m_CommandBufferSize;
        GLuint m_CommandRequestBeginId;
        GLsizei m_CommandRequestSize;
        u8* m_CommandBufferAddr;

        // Array for saving the vertex shader uniform matrix's 3D command buffer offset.
        detail::VertexShaderUniformMatrix m_VertexShaderUniformMatrixArray2d[DEMO_MAX_COMMAND_CACHE_MODELS_NUM][DEMO_MAX_COMMAND_CACHE_MATRIX_NUM];
    };

    void PrintCurrentCmdlist(void);

}

#endif
