// ----------------------------------------------------------------------
// File output class for debugging (for MGR)
// Main content
// - Outputs BMP files
// - Outputs return values
// Output destination is SD sdmc:/
// ----------------------------------------------------------------------
// Continuous output feature
// - Create new folder when initializing
// - When outputting, output BMP in number order and result text files into that folder
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>
#include	<nn/result.h>

#include	<string.h>

#include	"appdefine.h"

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"mgr.h"

#include	"CTR_Define.h"
#include	"DebugWriterMgr.h"

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::fs;
using namespace mw::mgr;

// ----------------------------------------------------------------------
// CDebugWriterMgr::CDebugWriterMgr
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CDebugWriterMgr::CDebugWriterMgr()
{
	m_dirNo		= 0;
	m_fileNo	= 0;
}

// ----------------------------------------------------------------------
// CDebugWriterMgr::~CDebugWriterMgr
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CDebugWriterMgr::~CDebugWriterMgr()
{
}

// ----------------------------------------------------------------------
// CDebugWriterMgr::Initialize
// ----------------------------------------------------------------------
// Content
//   Perform initialization for continuous output.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns true if initialization was normal.
// ----------------------------------------------------------------------
bool CDebugWriterMgr::Initialize()
{
	char szDirName[64];
	nn::fs::Directory dir;
	nn::Result result;

	int n = 0;
	for (n = 0 ; n < 100 ; n++)
	{
		sprintf(szDirName, "sdmc:/MGRLIB_%04d",n);
		result = dir.TryInitialize(szDirName);
		if (result.IsSuccess() == true)
		{
			dir.Finalize();
		}
		else
		{
			break;
		}
	}

	if (n >= 100)
	{
		return false;
	}

	result = nn::fs::TryCreateDirectory(szDirName);
	if (result.IsSuccess() == true)
	{
		m_dirNo = n;
		return true;
	}

	m_dirNo = 0;
	return false;
}

// ----------------------------------------------------------------------
// CDebugWriterMgr::Output
// ----------------------------------------------------------------------
// Content
//   Performs continuous output.
// ----------------------------------------------------------------------
// Arguments
//   rgbBuff		[ I ] Specifies the image buffer (RGB32).
//   width			[ I ] Specifies the image width.
//   height			[ I ] Specifies the image height.
//   result			[ I ] Specifies the detection results.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CDebugWriterMgr::Output(unsigned char* rgbBuff, int width, int height, MGResult* result)
{
	char szBmpName[64];
	char szResultName[64];
	nn::fs::Directory dir;

	sprintf(szBmpName,		"MGRLIB_%04d/%04d.bmp", m_dirNo, m_fileNo);
	sprintf(szResultName,	"MGRLIB_%04d/%04d.txt", m_dirNo, m_fileNo);

	OutputBmpImage(szBmpName, rgbBuff, width, height);
	OutputResult(szResultName, result);

	m_fileNo++;
}

// ----------------------------------------------------------------------
// CDebugWriterMgr::OutputBmpImage
// ----------------------------------------------------------------------
// Content
//   Outputs image files.
// ----------------------------------------------------------------------
// Arguments
//   strFileName	[ I ] Specifies the image file name.
//   rgbBuff		[ I ] Specifies the image buffer (RGB32).
//   width			[ I ] Specifies the image width.
//   height			[ I ] Specifies the image height.
// ----------------------------------------------------------------------
// Return value
//   Returns TRUE if output succeeds.
// ----------------------------------------------------------------------
bool CDebugWriterMgr::OutputBmpImage(char* strFileName, unsigned char* rgbBuff, int width, int height)
{
	char strFilePath[256];
	sprintf(strFilePath, "sdmc:/%s", strFileName);

	BMFH bmhf;
	BMIH bmih;

	bmhf.bfType = 0x4d42;
	bmhf.bfSize = 54 + width * height * 3;
	bmhf.bfRsv1 = 0;
	bmhf.bfRsv2 = 0;
	bmhf.bfBits = 54;

	bmih.biSize = 40;
	bmih.biWdth = width;
	bmih.biHigh = height;
	bmih.biPlan = 1;
	bmih.biBitc = 24;
	bmih.biComp = 0;
	bmih.biImag = width * height * 3;
	bmih.biXppm = 0;
	bmih.biYppm = 0;
	bmih.biCuse = 0;
	bmih.biCimp = 0;

	nn::fs::TryDeleteFile(strFilePath);
	nn::fs::FileOutputStream fos(strFilePath, true);

	fos.Write(&bmhf.bfType, sizeof(bmhf.bfType), true);
	fos.Write(&bmhf.bfSize, sizeof(bmhf.bfSize), true);
	fos.Write(&bmhf.bfRsv1, sizeof(bmhf.bfRsv1), true);
	fos.Write(&bmhf.bfRsv2, sizeof(bmhf.bfRsv2), true);
	fos.Write(&bmhf.bfBits, sizeof(bmhf.bfBits), true);

	fos.Write(&bmih.biSize, sizeof(bmih.biSize), true);
	fos.Write(&bmih.biWdth, sizeof(bmih.biWdth), true);
	fos.Write(&bmih.biHigh, sizeof(bmih.biHigh), true);
	fos.Write(&bmih.biPlan, sizeof(bmih.biPlan), true);
	fos.Write(&bmih.biBitc, sizeof(bmih.biBitc), true);
	fos.Write(&bmih.biComp, sizeof(bmih.biComp), true);
	fos.Write(&bmih.biImag, sizeof(bmih.biImag), true);
	fos.Write(&bmih.biXppm, sizeof(bmih.biXppm), true);
	fos.Write(&bmih.biYppm, sizeof(bmih.biYppm), true);
	fos.Write(&bmih.biCuse, sizeof(bmih.biCuse), true);
	fos.Write(&bmih.biCimp, sizeof(bmih.biCimp), true);

	u8* temp = (u8*)OS_MALLOC(bmih.biImag);
	u8* src  = rgbBuff;
	u8* dest = temp;

	for (int y = 0 ; y < bmih.biHigh ; y++)
	{
		for (int x = 0; x < bmih.biWdth ; x++)
		{
			src++;
			*dest++ = *src++;
			*dest++ = *src++;
			*dest++ = *src++;
		}
	}
	fos.Write(temp, bmih.biImag, true);
	fos.Finalize();

	OS_FREE(temp);
	temp = NULL;
	return true;
}

// ----------------------------------------------------------------------
// CDebugWriterMgr::OutputResult
// ----------------------------------------------------------------------
// Content
//   Outputs the detection results to a file.
// ----------------------------------------------------------------------
// Arguments
//   strFileName	[ I ] Specifies the output file name.
//   result			[ I ] Specifies the detection results.
// ----------------------------------------------------------------------
// Return value
//   Returns TRUE if output succeeds.
// ----------------------------------------------------------------------
bool CDebugWriterMgr::OutputResult(char* strFileName, MGResult* result)
{
	char strFilePath[256];
	sprintf(strFilePath, "sdmc:/%s", strFileName);

	int n = 0;

	nn::fs::TryDeleteFile(strFilePath);
	nn::fs::FileOutputStream fos(strFilePath, true);

	// ----------------------------------------------------------------------
	// ==================================================
	// 0000.bmp fingernum = 
	// --------------------------------------------------
	// No: [id] ( sx,  sy) - ( ex,  ey) ( cx,  cy)
	// ==================================================
	//
	// --------------------------------------------------
	// 0000.bmp fingernum = 5
	// --------------------------------------------------
	//  0: (xxx, yyy) zzz/aaa
	// ----------------------------------------------------------------------
	Write(fos, "==================================================\n", true);
	Write(fos, "bodynum = %d\n", result->bodynum, true);
	Write(fos, "--------------------------------------------------\n", true);
	Write(fos, "No: [id] ( sx,  sy) - ( ex,  ey) ( cx,  cy)\n", true);
	Write(fos, "==================================================\n", true);

	int num = result->bodynum;
	MGObject*		object	= result->mgobject;

	for (n = 0 ; n < num ; n++)
	{
		Write(fos, "%04d: [%02d] (%3d, %3d) - (%3d, %3d) (%3d, %3d)\n",
			n + 1,
			object->id,
			object->rect.sx,
			object->rect.sy,
			object->rect.ex,
			object->rect.ey,
			object->center.x,
			object->center.y,
			true);
		object++;
	}

	fos.Finalize();
	return true;
}

// ----------------------------------------------------------------------
// CDebugWriterMgr::Write
// ----------------------------------------------------------------------
// Content
//   Outputs a string to a file.
// ----------------------------------------------------------------------
// Arguments
//   fos		[ I ] Specifies the output stream.
//   fmt...		[ I ] Specifies the string format.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CDebugWriterMgr::Write(nn::fs::FileOutputStream& fos, char* fmt, ...)
{
	// format string process
    char str[256];
    va_list va;
    s32 stringSize;
    
    va_start(va, fmt);
    stringSize = vsnprintf(str, 256, fmt, va);
    va_end(va);

	fos.Write(str, stringSize, true);
}
